Attribute VB_Name = "modImageProcess"
'************************************************************************************
'Purpose: This function takes a variable and returns it's address.
'         Basically, it is the C++ '&' (adress) command
'************************************************************************************
Private Declare Function GetAddress Lib "converts.dll" ( _
            ByRef data As Byte) As Long

Option Explicit
            
            
Public Function Get_Picture(Video_Control As VideoPortal, data() As Byte, ByRef Pic_Size As Long) As Boolean
'************************************************************************************
'Purpose: This captures an image from the camera as puts it in the data() array.
'         Uses PictureToMemory
'
'
'Inputs:
'       VideoPortal - A Logitech control to the camer
'       data() - A storage area of bytes for the captured image
'       Pic_Size - How big is the actual image?
'************************************************************************************

                          
    Dim Ret_Value As Long   'Returns a 1 or 0, depending on functions success
    Dim Pic_Address As Long 'This is the address of the first element of the data
                            
    On Error GoTo GetPictureError
       
    '2 calls need to be made to the picture to memory function.  The first call
    'gets the amount of space needed to store the BMP data.  For the camera we used
    '(Logitech QuickCam Pro 3000), there are 3 sizes of images, each requiring a
    'different amount of space.
    Ret_Value = Video_Control.PictureToMemory(0, 24, 0, Pic_Size, "")
    
    'This statement is needed to setup the data array based on the Pic_Size
    ReDim data(Pic_Size)
    
    'This statement gets the address of the first element of the data array
    Pic_Address = GetAddress(data(0))
    
    'The 0 and the 24 are required by the Logitech driver.
    'Pic_Adress and Pic_Size are where to store and how much to store
    'the "" value allows for a text overlay.  I do not use it.
    Ret_Value = Video_Control.PictureToMemory(0, 24, Pic_Address, Pic_Size, "")
    
    Get_Picture = True 'Success
    
Exit Function

GetPictureError:
    Get_Picture = False 'Failure
End Function


Public Function Process_Picture(Picture_Box As PictureBox, Picture_Target As PictureBox, data() As Byte, ByRef Pic_Size As Long, MatrixArray() As Long, _
                                ByRef AvgRed As Double, ByRef AvgGreen As Double, ByRef AvgBlue As Double, Form As Form)
'************************************************************************************
'Purpose: This function runs through the entire image and stars the totals
'         calculations.
'
'
'Inputs:
'       Picture_Box - Where the preview should go
'       Picture_Box - Where the RED target area should go
'       data() - A storage area for the captured image from the camera
'       Pic_Size - How much space is needed for the picture?
'       Matrix_Array - Where should I store the totals? Needs to be an array
'       avgRED - Average number of all RED pixels for auto-calibration
'       avgGREEN - Average number of all GREEN pixels for auto-calibration
'       avgBlue - Average number of all BLUE pixels for auto-calibration
'       Form - What form is this stuff on?
'************************************************************************************


    Dim x As Integer        'The x-coordinate of the image
    Dim y As Integer        'The y-coordinate of the image
    
    Dim RGB_Offset As Long  'This is the number of bytes used for the header.
                            'Needed to get to the actual BMP data.
                            
    Dim i As Integer        'A runner to go through the picture array's bytes
                            
    RGB_Offset = 54         'There are 55 bytes needed for the header information.
                            'This is a constant for our images.  Less header is needed
                            'for a 24-bit image since the color information does not
                            'need to be saved.
                            
    x = 0                   'Start off at x = 0
    y = 120                 'BMP files are stored upside-down, so reading must begin
                            'from the bottom.
                            
    Dim cRed As Long        'Current RED value of target pixel
    Dim cGreen As Long      'Current GREEN value of target pixel
    Dim cBlue As Long       'Current BLUE value of target pixel
    Dim cBuf As Long        'Current BUFFER value for target pixel
    Dim totPixels As Integer 'Running total of pixel processed
                            
    cRed = CLng(Form.RED.Text) 'Store the RED value
    cGreen = CLng(Form.GREEN.Text) 'Store the GREEN value
    cBlue = CLng(Form.BLUE.Text) 'Store the BLUE value
    cBuf = CLng(Form.BUF.Text) 'Store the BUFFER value


    For i = 1 To (Pic_Size - 54) / 3
        'This is a very slow function that sets each pixel on the screen.  While it is
        'slow, it is fast enough for the testing program.  The data is stored GRB
        Picture_Box.PSet (x, y), RGB(data(RGB_Offset + 2), data(RGB_Offset + 1), data(RGB_Offset))
        
        'This large if block test to see if we found a traget pixel, if we did it
        'Marks it in Matrix_Array, sets it's location to red, adds it to the averages
        'And increased total number of found pixels by 1
        If (data(RGB_Offset + 2) >= cRed - cBuf) And (data(RGB_Offset + 2) <= cRed + cBuf) Then
           If (data(RGB_Offset + 1) >= cGreen - cBuf) And (data(RGB_Offset + 1) <= cGreen + cBuf) Then
                If (data(RGB_Offset) >= cBlue - cBuf) And (data(RGB_Offset) <= cBlue + cBuf) Then
                    AvgRed = AvgRed + data(RGB_Offset + 2)
                    AvgGreen = AvgGreen + data(RGB_Offset + 1)
                    AvgBlue = AvgBlue + data(RGB_Offset)
                    totPixels = totPixels + 1
                    MatrixArray((x / 10), (y / 7)) = MatrixArray((x / 10), (y / 7)) + 1
                        
                    'This PSet just sets RED pixels where we find target pixels.
                    Picture_Target.PSet (x, y), RGB(255, 0, 0)
                End If
            End If
        End If
            
        'Add 3 to get the next pixel
        RGB_Offset = RGB_Offset + 3
        
        'If we are at the end of the line, start again, otherwise, go to the next pixel.
        If (i Mod (160)) = 0 Then
           x = 0
           y = y - 1
        Else
           x = x + 1
        End If
    Next i
    
    'If we actually found pixel, go ahead and calculate the averages
    If Not ((totPixels = 0)) Then
        AvgRed = CInt(AvgRed / totPixels)
        AvgGreen = CInt(AvgGreen / totPixels)
        AvgBlue = CInt(AvgBlue / totPixels)
    End If
    
    totPixels = 0
    
End Function


Public Function Setup_Winsock(WinSock As WinSock) As Boolean
'************************************************************************************
'Purpose: This function initializes and sets up the WinSock control for communication
'
'
'Inputs:
'       WinSock - A WinSock control
'
'************************************************************************************

WinSock.Protocol = sckTCPProtocol 'We want to use TCP protocol
   
   'Initialize the traget to be localhost
   WinSock.RemoteHost = ""
   WinSock.RemoteHost = _
      InputBox("Enter the remote host IP address", _
               "IP Address", "localhost 138.74.8.215")
   
   'If none was entered, simply use ourself (localhost)
   If WinSock.RemoteHost = "" Then
      WinSock.RemoteHost = "localhost"
   End If
   
   'Pick an arbitrary port to use assuming the sever is on the same port.
   'Also, don't use a port that something else is using.
   WinSock.RemotePort = 3900
   
   Call WinSock.Connect 'Go ahead and make the connection

End Function


Public Function Send_Command(WinSock As WinSock, Command As String)
'************************************************************************************
'Purpose: This takes a string and sends it to the server as a command
'
'
'Inputs:
'       WinSock - The WinSock control
'       Command - The command to send (Following server protocol)
'
'************************************************************************************
    WinSock.SendData (Command)
End Function

Public Function Clear_MatrixArray(MatrixArray() As Long)
'************************************************************************************
'Purpose: Simple function to clear out the Matrix Array for processing the next image
'
'Inputs:
'       MatrixArray - Array to clear
'
'
'************************************************************************************

    
    Dim xCount As Integer 'Runner for X-Axis
    Dim yCount As Integer 'Runner for Y-Axis

    'Needs to be 18,18 for proper processing
    'Run through each one, and set it to 0
    For xCount = 0 To 18
        For yCount = 0 To 18
            MatrixArray(xCount, yCount) = 0
        Next yCount
    Next xCount
    
End Function


Public Function Decide_Direction(cMaxQuadX As Integer)
'************************************************************************************
'Purpose: Decide which command to send to the server for turning.  Also, set various
'         visuals for the user to know where we are going
'
'Inputs:
'      cMaxQuadX - Which section has the largest number of target pixels?
'
'
'************************************************************************************

    Select Case cMaxQuadX
        'If it is from -1 --> +1, remain on course
        Case -1:
            frmProcess.txtDirection.Text = "Remain Course:" & cMaxQuadX
            frmProcess.BackColor = &H800080 'Set BG to purple to signify
                                            'acceleration
        Case 0:
            frmProcess.txtDirection.Text = "Remain Course:" & cMaxQuadX
            frmProcess.BackColor = &H800080 'Set BG to purple to signify
                                            'acceleration
        Case 1:
            frmProcess.txtDirection.Text = "Remain Course:" & cMaxQuadX
            frmProcess.BackColor = &H800080 'Set BG to purple to signify
                                            'acceleration
        'If it is greater than 1, turn right
        Case Is > 1:
            frmProcess.txtDirection.Text = "Turn Right:" & cMaxQuadX
            Call Send_Command(frmProcess.Winsock1, "*ROBOT*TR1")
            Call frmProcess.WaitCMD 'Call this so we have to wait for ACK
            frmProcess.BackColor = &H80FF80 'Set BG to green to signify
                                            'halting until target centered
        'If it is less than 1, turn left
        Case Is < 1:
            frmProcess.txtDirection.Text = "Turn Left:" & cMaxQuadX
            Call Send_Command(frmProcess.Winsock1, "*ROBOT*TL1")
            Call frmProcess.WaitCMD 'Call this so we have to wait for ACK
            frmProcess.BackColor = &H80FF80 'Set BG to green to signify
                                            'halting until target centered
    End Select

End Function

Public Function Process_Matrix(MatrixArray() As Long)
'***********************************************************************************
'Purpose: This function loops through the Matrix of target pixel totals to compute
'         what should be done next
'
'
'***********************************************************************************

    Dim yCount As Integer 'A y runner for Matrix processing
    Dim xCount As Integer 'A x runner for Matrix processing
    
    Dim cMaxQuad As Integer 'Which section has the most target pixel?
    Dim cMaxQuadX As Integer 'A running total of the most target pixel

    cMaxQuad = 0 'Initialize to 0
    cMaxQuadX = 0 ' Initialize to 0
    
    'Run through the entire Matrix in a systematic way making totals each step.
    'This totals each vertical section
    For xCount = 1 To 18
        For yCount = 1 To 18
            MatrixArray(xCount, 0) = MatrixArray(xCount, 0) + MatrixArray(xCount, yCount)
        Next yCount
    Next xCount
    
    'This one totals each subtotal and find the section with the most target
    'pixel.  This is the last calculation that needs to be performed before
    'we can decide where to go
    For xCount = 1 To 18
        If (MatrixArray(xCount, 0) > cMaxQuad) Then
            cMaxQuad = MatrixArray(xCount, 0)
            cMaxQuadX = xCount
        End If
    Next xCount
    
    'We didn't find any of the target pixel
    If cMaxQuadX = 0 Then
        cMaxQuadX = -10
    Else
    'We found target pixel, so subtract 8 to make the center
    'of the image 0
        cMaxQuadX = cMaxQuadX - 8
    End If
    
    'Now we have the section with the most target; we need to move somewhere
    Call Decide_Direction(cMaxQuadX)
    
End Function

